@@ -16,27 +16,31 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
+#ifndef GNUTLS_LIB_AUTH_SRP_PASSWD_H
+#define GNUTLS_LIB_AUTH_SRP_PASSWD_H
+
 #ifdef ENABLE_SRP
 
-typedef struct
-{
-  char *username;
+typedef struct {
+	char *username;
 
-  gnutls_datum_t salt;
-  gnutls_datum_t v;
-  gnutls_datum_t g;
-  gnutls_datum_t n;
+	gnutls_datum_t salt;
+	gnutls_datum_t v;
+	gnutls_datum_t g;
+	gnutls_datum_t n;
 } SRP_PWD_ENTRY;
 
 /* this is locally allocated. It should be freed using the provided function */
-int _gnutls_srp_pwd_read_entry (gnutls_session_t state, char *username,
-                                SRP_PWD_ENTRY **);
-void _gnutls_srp_entry_free (SRP_PWD_ENTRY * entry);
-int _gnutls_sbase64_decode (char * data, size_t data_size,
-                            uint8_t ** result);
+int _gnutls_srp_pwd_read_entry(gnutls_session_t state, char *username,
+			       SRP_PWD_ENTRY **);
+void _gnutls_srp_entry_free(SRP_PWD_ENTRY * entry);
+int _gnutls_sbase64_decode(char *data, size_t data_size,
+			   uint8_t ** result);
 
-#endif /* ENABLE_SRP */
+#endif				/* ENABLE_SRP */
+
+#endif /* GNUTLS_LIB_AUTH_SRP_PASSWD_H */
